import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { CommandHandler } from '@nestjs/cqrs';
import { ConfigurationMessagerPresenter } from '../../adapters/secondaries/configuration-messager.presenter';
import { ConfigurationMessager } from '../../adapters/secondaries/configuration.messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';
import { UpdateConfigurationCommand } from '../../commands/update-configuration.command';
import { UpdateConfigurationRequest } from '../dtos/update-configuration.request';
import { Configuration } from '../entities/configuration';

@CommandHandler(UpdateConfigurationCommand)
export class UpdateConfigurationUseCase {
  constructor(
    private readonly _repository: ConfigurationRepository,
    private readonly _configurationMessager: ConfigurationMessager,
    private readonly _loggingMessager: LoggingMessager,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  async execute(command: UpdateConfigurationCommand): Promise<Configuration> {
    const entity = this._mapper.map(
      command.updateConfigurationRequest,
      UpdateConfigurationRequest,
      Configuration,
    );

    try {
      const configuration = await this._repository.update(
        command.updateConfigurationRequest.uuid,
        entity,
      );
      this._configurationMessager.publish(
        'update',
        JSON.stringify(
          this._mapper.map(
            configuration,
            Configuration,
            ConfigurationMessagerPresenter,
          ),
        ),
      );
      this._loggingMessager.publish(
        'configuration.update.info',
        JSON.stringify(command.updateConfigurationRequest),
      );
      return configuration;
    } catch (error) {
      this._loggingMessager.publish(
        'configuration.update.crit',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
